/* --- Copyright University of Sussex 1996. All rights reserved. ----------
 > File:            C.unix/lib/ved/ved_print.p
 > Purpose:         Print files from VED
 > Author:          Mark Rubinstein, Sep 17 1985 (see revisions)
 > Documentation:   HELP * VED_PRINT
 > Related Files:   C.vms/lib/ved/ved_print.p
 */

compile_mode :pop11 +strict;

section;

uses postscript_line_consumer;

/* ==== VED_PRINT INTERFACE ============================================ */

;;; writes a range of a ved buffer to a file, converting it to PostScript
define lconstant PS_write_range(device, lo, hi);
	lvars device, lo, hi, file, descr;

	device_open_name(device) -> file;

	;;; write all the lines from lo to hi
	if lo <= vedline and vedline <= hi then vedtrimline() endif;
	min(vedusedsize(vedbuffer), hi) -> hi;

	if hi == 0 then
		;;; buffer is empty
		if file then
			file sys_>< '(EMPTY)' -> descr;
			vedputmessage(descr)
		endif;
		return;
	else
		if lo > hi then
			mishap(lo, hi, 2, 'IMPOSSIBLE RANGE TO WRITE')
		endif;
		if file then
			file sys_>< ', ' sys_>< (hi - lo + 1) sys_>< ' LINES' -> descr;
			vedputmessage('WRITING ' sys_>< descr)
		endif
	endif;

	lvars procedure consume = postscript_line_consumer(device);

	while lo <= hi do
		;;; write next line
		consume(subscrv(lo, vedbuffer));
		lo + 1 -> lo
	endwhile;

	consume(termin);

	if file then
		vedputmessage(descr sys_>< ' WRITTEN')
	endif;
enddefine;

;;; writes contents of file to a device, converting it to PostScript
define lconstant PS_write_file(device, file);
	lvars device, file,
		procedure rep = vedfile_line_repeater(file),
		procedure consume = postscript_line_consumer(device),
		procedure filter = rep <> dup <> consume,
	  ;
	until filter() == termin do enduntil;
enddefine;


lvars
	command,    ;;; print command to use: lpr(1) or lp(1)
	using_lpr,  ;;; <true> if -command- is lpr
	files,      ;;; list of files to print
	printer,    ;;; printer to use
	filters,    ;;; list of pre-filters
	copies,     ;;; number of copies to print
	flags,      ;;; other flags to pass
	devtype,    ;;; "ps" if it is postscript - false otherwise
	;

	;;; choose which print command to use
define lconstant setup();
	returnunless(isundef(command));
	lconstant PATH = systranslate('$PATH');
	if (sys_search_unix_path('lp', PATH) ->> command) then
		false -> using_lpr
	elseif (sys_search_unix_path('lpr', PATH) ->> command) then
		true -> using_lpr
	else
		vederror('print: can\'t find print command (lp/lpr)');
	endif
enddefine;

	;;; convert ved command flags into corresponding lpr/lp options
define lconstant translate_flags();
	consstring(#|
		if strmember(`m`, flags) then explode('-m ') endif;
		if using_lpr then
			if strmember(`f`, flags) then explode('-h ') endif;
			if strmember(`h`, flags) then explode('-p ') endif;
			if strmember(`l`, flags) then explode('-l ') endif;
		else
			if strmember(`f`, flags) or strmember(`l`, flags) then
				vederror('print: lp(1) doesn\'t support flags -f & -l');
			endif;
			if strmember(`h`, flags) then [^^filters 'pr '] -> filters endif;
			;;; always add '-s' to suppress messages
			explode('-s ');
		endif;
	|#) -> flags;
enddefine;

	;;; construct a shell command to do the printing
define lconstant gen_print_command(file, title);
	lvars file, title;
	consstring(#|
		unless filters == [] then
			lvars filter;
			if file then
				explode('cat '), explode(file), explode(' | ');
				unless title then file -> title endunless;
				false -> file;
			endif;
			for filter in filters do
				explode(filter), explode(' | ');
			endfor;
		endunless;
		explode(command), ` `;
		explode(flags);
		if copies > 1 then
			explode(if using_lpr then '-#' else '-n' endif);
			dest_characters(copies), ` `;
		endif;
		unless printer = nullstring then
			explode(if using_lpr then '-P' else '-d' endif);
			explode(printer), ` `;
		endunless;
		if file then
			explode(file);
		elseif title then
			if using_lpr then
				explode('-J '), explode(title);
				explode(' -T '), explode(title);
			else
				explode('-t'), explode(title);
			endif;
		endif;
	|#);
enddefine;

	;;; split -vedargument- into flags, filters and files
define lconstant parseargument();
	lconstant SPECIALS = '-#.$~/:', FLAGS = 'fhlmd';
	lvars item, procedure items = incharitem(stringin(vedargument)), nesting;

	;;; modify the itemiser to treat SPECIAL characters as alphabetic
	appdata(
		SPECIALS,
		procedure(c);
			lvars c;
			1 -> item_chartype(c, items);
		endprocedure);

	until (items() ->> item) == termin do
		if isinteger(item) then
			item -> copies;
		elseif isstring(item) then
			;;; quoted file name
			[^^files ^item] -> files;
		elseif isword(item) and isstartstring('-', item) then
			;;; parse flags
			lvars i, c, n = 0;
			for i from 2 to datalength(item) do
				subscrw(i, item) -> c;
				if c == `p` then
					if (items() ->> printer) == termin then
						vederror('print: printer name expected after -p flag');
					endif;
				elseif c == `o` then
					if (items() ->> devtype) == termin then
						vederror('print: device output type expected after -p flag');
					endif;
					if devtype /== "ps" then
						vederror('print: unknown device type');
					endif;
				elseif c == `#` then
					;;; ignore
				elseif isnumbercode(c) then
					;;; number of copies
					n * 10 + c - `0` -> n;
				elseif strmember(c, FLAGS) then
					flags <> consstring(c,1) -> flags;
				else
					vederror('print: unrecognized flag - ' <> consstring(c,1));
				endif;
			endfor;
			unless n == 0 then n -> copies endunless;
		elseif item == "(" then
			;;; read a filter
			1 -> nesting;
			consstring(#|
				repeat
					nextchar(items) -> item;
					if item == termin then
						vederror('print: closing ) not found')
					elseif item == `(` then
						nesting fi_+ 1 -> nesting;
					elseif item == `)` then
						nesting fi_- 1 -> nesting;
						quitif(nesting == 0)
					endif;
					item
				endrepeat;
			|#) -> item;
			[^^filters ^item] -> filters;
		else
			[^^files ^(item sys_>< nullstring)] -> files;
		endif;
	enduntil;
	translate_flags();
enddefine;


define lconstant do_pipeout(source, comm);
	lvars source, comm;
	pipeout(
		source,
		if strmember(`|`, comm) then
			'/bin/sh', ['sh' '-c' ^comm]
		else
			dest(sysparse_string(comm))
		endif,
		true)
enddefine;

	;;; print a range of the current buffer
define lconstant vedprintmr(lo, hi);
	lvars lo, hi;
	dlocal vednotabs = true;    ;;; to give correct spacing
	unless hi >= lo then vederror('print: nothing to print') endunless;
	do_pipeout(
		if devtype == "ps" then
			consref(PS_write_range(% lo, hi %)),
		else
			consref(vedwriterange(% lo, hi %)),
		endif,
		gen_print_command(false, sys_fname_name(vedcurrent)))
enddefine;

define lconstant vedprint(marked_range);
	lvars marked_range;
	dlocal
		devtype = false,
		files   = [],
		printer = systranslate('popprinter') or nullstring,
		filters = [],
		copies  = 1,
		flags   = nullstring;
	setup();
	parseargument();

	if marked_range == "ps" then
		if files == [] then
			mishap(0, 'PSPRINT - no files specified');
		endif;
		"ps" -> devtype;
		false -> marked_range;
	endif;

	if files == [] then
		vedprintmr(
			if marked_range then
				vedputmessage('printing marked range ...');
				vvedmarklo, vvedmarkhi
			else
				vedputmessage('printing whole file ...');
				1, vvedbuffersize
			endif);
	elseif marked_range then
		vederror('printmr: only works on current file');
	else
		vedputmessage('printing named file(s) ...');
		lvars file;
		for file in files do
			if devtype == "ps" then
				do_pipeout(
					consref(PS_write_file(% file %)),
					gen_print_command(false, sys_fname_name(file)))
			else
				sysobey(gen_print_command(file, false))
			endif
		endfor;
	endif;
	vedputmessage('print command queued');
enddefine;

define global vars ved_printmr =
	vedprint(% true %);
enddefine;

define global vars ved_print =
	vedprint(% false %);
enddefine;

;;; used by vedprint command to parse command line arguments
define global vars vedpsprint;
	dlocal vedargument;

	;;; convert poparglist into a string
	consstring(#| procedure;
		dlocal cucharout = identfn;
		applist(poparglist, spr)
	endprocedure() |#) -> vedargument;
	vedprint("ps");
enddefine;


endsection;


/* --- Revision History ---------------------------------------------------
--- John Williams, Jan  4 1996
		Now selects __lp in preference to ___lpr, if possible (cf. BR adrianh.69)
		Also, doesn't run /bin/sh unless necessary.
--- John Williams, Aug  2 1994
		______________parse_argument now copes with nested ( and ). (cf. BR adrianh.60)
--- Jonathan Meyer, Oct 12 1993
		Added '-o ps' option and PS_ procedures for printing to postscript.
--- John Gibson, Jul 22 1992
		Made vednotabs true inside vedprintmr so printing is done with
		correct spacing.
--- Robert John Duncan, Jun 25 1992
		Merged BSD and System V versions and moved to C.unix
--- John Gibson, Mar 10 1992
		Uses -vedwriterange- with -pipeout-. Cleaned up.
--- James Goodlet, Jun  6 1989 - pipeout told to wait (4th arg true), so that
		large processes can print files without running out of swap.
--- John Williams, Apr 17 1986 - fixed printing of files other than current
	file.  Needed brackets around <> in flag mashing.
*/
